///////////////////////////////////////////////////////////////////////////////
// filename:    wString.uc
// revision:    100
// authors:     Michiel 'El Muerte' Hendriks <elmuerte@drunksnipers.com>
//              various UnrealWiki members (http://wiki.beyondunreal.com)
///////////////////////////////////////////////////////////////////////////////

class wString extends Object;

static final function string StrShift(out string line, string delim)
{
  local int pos;
  local string result;

  pos = Instr(line, delim);
  if (pos == -1)
  {
    result = line;
    line = "";
  }
  else {
    result = Left(line,pos);
    line = Mid(line,pos+1);
  }
  return result;
}

static final function string StrReplace(coerce string target, array<string> replace, array<string> with)
{
  local int i,j;
	local string Input;

  for (j = 0; j < replace.length; j++)
  {
  	Input = target;
	  target = "";
  	i = InStr(Input, Replace[j]);
	  while(i != -1)
  	{	
	  	target = target $ Left(Input, i) $ With[j];
		  Input = Mid(Input, i + Len(Replace[j]));	
  		i = InStr(Input, Replace[j]);
	  }
  	target = target $ Input;
  }
  return target;
}

static final function string Lower(coerce string Text) 
{
  local int IndexChar;
  for (IndexChar = 0; IndexChar < Len(Text); IndexChar++)
    if (Mid(Text, IndexChar, 1) >= "A" &&
        Mid(Text, IndexChar, 1) <= "Z")
      Text = Left(Text, IndexChar) $ Chr(Asc(Mid(Text, IndexChar, 1)) + 32) $ Mid(Text, IndexChar + 1);
  return Text;
}

static final function bool IsUpper(coerce string S)
{
    return S == Caps(S);
}

static final function bool IsLower(coerce string S)
{
    return S == Lower(S);
}

static final function string LTrim(coerce string S)
{
    while (Left(S, 1) == " ")
        S = Right(S, Len(S) - 1);
    return S;
}

static final function string RTrim(coerce string S)
{
    while (Right(S, 1) == " ")
        S = Left(S, Len(S) - 1);
    return S;
}

static final function string Trim(coerce string S)
{
    return LTrim(RTrim(S));
}

static final function bool _match(out string mask, out string target)
{
  local string m;
  if (mask == "") return true;
  m = Left(mask,1);
  if (m == "*") 
  {
    mask = Mid(mask, 1);
    return _matchstar(m, mask, target);
  }
  if (Len(target) > 0 && (m == "?" || m == Left(target,1)) ) 
  {
    mask = Mid(mask, 1);
    target = Mid(target, 1);
    return _match(mask, target);
  }
  return false;
}

static final function bool _matchstar(string m, out string mask, out string target)
{
  local int i, j;
  local string t;

  if (mask == "") return true;

  for (i = 0; (i < Len(target)) && (m == "?" || m == Mid(target, i, 1)); i++)
  {
    j = i;
    do {
      t = Left(target, j);
      if (_match(mask, t)) return true;
    } until (j-- <= 0)
  }
  return false;
}

// Wildcards: * = X chars; ? = 1 char
// Wildcards can appear anywhere in the mask
static final function bool MaskedCompare(coerce string target, string mask, optional bool casesensitive)
{
  if (!casesensitive)
  {
    mask = Caps(mask);
    target = Caps(target);
  }
  if (mask == "*") return true;

  do {
    if ( _match(mask, target)) return true;
    target = Mid(target, 1);
  } until (Len(target) <= 0);
  return false;
}

static final function int InStrFrom(coerce string StrText, coerce string StrPart, optional int OffsetStart)
{
  local int OffsetPart;

  OffsetPart = InStr(Mid(StrText, OffsetStart), StrPart);
  if (OffsetPart >= 0)
    OffsetPart += OffsetStart;
  return OffsetPart;
}


static function bool ReplaceOption( out string Options, string Key, string NewValue, optional bool bAddIfMissing, optional out string OldValue, optional string delim)
{
  local array<string> OptionsArray;
  local int i;
  local string   CurrentKey, CurrentValue;
  local bool     bReplaced;
  bReplaced = false;

  if (delim == "") delim = "?";
  // Need to strip the first ? from Options otherwise Split doesn't work
  if ( Left( Options, 1 ) == delim )
    Options = Right( Options, Len( Options ) - 1 );

  Split( Options, delim, OptionsArray );

  for ( i = 0; i < OptionsArray.Length; i++ ) {
    Divide( OptionsArray[i], "=", CurrentKey, CurrentValue );

    if ( CurrentKey ~= Key ) {
      OldValue = CurrentValue;
      OptionsArray[i] = CurrentKey$"="$NewValue;
      bReplaced = true;
    }
  }

  if ( !bReplaced && bAddIfMissing )
    OptionsArray[OptionsArray.Length] = Key$"="$NewValue;

  Options = delim$class'wArray'.static.Join(OptionsArray, delim);
  return bReplaced;
}
